

(**************************************************************************)
(* Utilities *)

open Data
let get_float var sol =  
  match (try List.assoc var sol with Not_found -> 
    Printf.printf "*** can not find the value of %s\n" var;
    flush stdout;
    assert false) 
  with (F f) -> f | _ -> 
    Printf.printf "*** error when parsing float\n";
    flush stdout;
    assert false
 
let get_bool var sol =  
  match (try List.assoc var sol with Not_found -> 
    Printf.printf "*** can not find the value of %s\n" var;
    flush stdout;
    assert false) 
  with (B b) -> b | _ -> 
    Printf.printf "*** error when parsing bool\n";
    flush stdout;
    assert false

(**************************************************************************)

let init  _ = ()
let kill  _ = ()
let mems_i = []
let mems_o = []

let (inputs  :(string * Data.t) list) = [ "a", Data.Real; "b", Data.Real]
let (outputs :(string * Data.t) list) = [ "x", Data.Real; "y", Data.Real]

let first_step = ref true
let (step :  Data.subst list -> Data.subst list)=
  fun env_outs -> 
    if !first_step  then  (
      (* ZZZ The env starts, and thus should be able to perform its
         first step with no input! *)
      first_step := false;
      [("x", F 10.0);("y", F 10.0)]
    )
    else 
      let a = get_float "a" env_outs
      and b = get_float "b" env_outs in
      let x = a +. 1.0 and y = b +. 1.0 in
(*       Printf.printf "Env step x=%f y=%f a=%f b=%f\n" x y a b; *)
(*       flush stdout; *)
      [("x",F x);("y",F y)]


type ctx = Event.t
type e = Event.t

let (step_dbg : Data.subst list -> ctx -> (Data.subst list -> ctx -> e) -> e) =
  fun sl ctx cont -> 
  let nctx = Event.incr_event_nb ctx in
  let cont2 sl ctx =
    let nctx = Event.incr_event_nb ctx in
    {
      Event.step = ctx.Event.step;
      Event.data = sl;
      Event.nb = ctx.Event.nb;
      Event.depth = ctx.Event.depth;
      Event.kind = Event.Exit;
      Event.lang = "ocaml";
      Event.name = "ze env";
      Event.inputs = inputs ;
      Event.outputs = outputs;
      Event.locals = [];
      Event.sinfo = None; 
      Event.next = (fun () -> cont sl nctx);
      Event.terminate = ctx.Event.terminate;
    }
  in
  {
    Event.step = ctx.Event.step;
    Event.data = ctx.Event.data;
    Event.nb = ctx.Event.nb;
    Event.depth = ctx.Event.depth;
    Event.kind = Event.Call;
    Event.lang = "ocaml";
    Event.name = "ze env";
    Event.inputs = inputs ;
    Event.outputs = outputs;
    Event.locals = [];
    Event.sinfo = None; (* cf sut.ml for a possible way to fill that in *)

    Event.next = (fun () -> cont2 (step sl) nctx);
    Event.terminate = ctx.Event.terminate;
  }

      
open RdbgPlugin 
let plugin = {
  id = "ocaml env";
  inputs = inputs;
  outputs= outputs;
  kill= kill;
  init_inputs= mems_i;
  init_outputs= mems_o;
  step = step;
  step_dbg = step_dbg
}

