(* Time-stamp: <modified the 28/03/2018 (at 10:07) by Erwan Jahier> *)

(* nb : comments preceeded by 2 "*" are used in the online doc  *)

(** Compute the next event, call the hooks functions on it, and return that event *)
val next : Event.t -> Event.t
val next_np : Event.t -> Event.t


(** Add a function that is called after each next call *)
val add_hook : string -> (Event.t -> unit) -> unit

(** Get a hook function (raise Not_found if it does not exist) *)
val get_hook : string -> (Event.t -> unit) 

(** Remove a hook  *)
val del_hook : string -> unit

(** Remove all hooks *)
val del_all_hooks : unit -> unit

(** List hooks *)
val list_hooks : unit -> string list

(** Returns the ith next event *)
val nexti : Event.t -> int -> Event.t

(** Returns the ith next event without printing events *)
val nexti_np : Event.t -> int -> Event.t

(** go to the next simulation step of the current node *)
val step : Event.t -> Event.t

(** go to the ith simulation step of the current node *)
val stepi : Event.t -> int -> Event.t

(** go to step number i  *)
val goto_s : Event.t -> int -> Event.t

(** loop forever *)
val loopforever : Event.t -> unit

(** control the printing of skipped events *)
val show_trace : bool ref

(** control the behavior of print_event *)
val show_data : bool ref

(** control the behavior of print_event *)
val show_src : bool ref

(** Print an event *)
val print_event : Event.t -> unit

(** Returns the next event that satisfies a condition *)
val next_cond : Event.t -> (Event.t -> bool) -> Event.t

(** Get the value of an int variable *)
val v : string -> Event.t -> Data.v

(** Get the value of an int variable *)
val vi : string -> Event.t -> int

(** Get the value of a float variable *)
val vf : string -> Event.t -> float

(** Get the value of a boolean variable *)
val vb : string -> Event.t -> bool

  
(** Print some source code information attached to an event *)
val print_src : Event.t -> unit

(** print the call stack *)
val print_call_stack : Event.t -> unit

                             
(** add a breapoint on a node or a file.
A breakpoint is a string of the form:
       "node"
       "node::line"
       "file"
       "file::line"
 *)
val break : string -> unit

(** Remove all breakpoints *)
val delete : unit -> unit

(** continue to the next breakpoint  *)
val continue : Event.t -> Event.t

(** Goto to a specific explicit breakpoint (cf 'break' for breakpoints syntax) *)
val goto_brk : Event.t -> string -> Event.t


(** go to event nb i *)
val goto : Event.t -> int -> Event.t



(* move backwards *)

(** turns on/on the possibility to run backwards *)
val time_travel : bool -> unit

(** checkpoint rate *)
val ckpt_rate : int ref

(** run backwards until the next breakpoint *)
val rev : Event.t -> Event.t

(** run backwards until the next breakpoint or condition *)
val rev_cond : Event.t -> (Event.t -> bool) -> Event.t

(** move one event back *)
val back : Event.t -> Event.t
(** move i events back *)
val backi : Event.t -> int -> Event.t

                                                 
val run : unit -> Event.t


