(* Time-stamp: <modified the 03/05/2018 (at 11:14) by Erwan Jahier> *)

open Event

let rdbg_tuning_file = "my-rdbg-tuning.ml"

let pversion () =
  Printf.fprintf stdout "%s (%s)\n" RdbgVersion.str RdbgVersion.sha;
  flush stdout;
  exit 0

(* exported *)
let verbose = ref false
let lurette = ref false
let suts = ref []
let envs = ref []
let oracles = ref []
let suts_nd = ref []
let envs_nd = ref []
let oracles_nd = ref []
let test_length = ref 100
let output_file = (* what is the better default name ?? *)
  ref (Filename.basename (Sys.getcwd()) ^".rif")
let drdbg = ref false
let options = ref []
let arg_options = ref []
let emacs_mode = ref false
let display_gnuplot = ref false
let display_sim2chro = ref false
let margin = ref 6 
let usage = ref ""
                 
let pspec os (c, ml) = (
  let (m1, oth) = match ml with
	 |	h::t -> (h,t)
	 |	_ -> ("",[])
  in
  let t2 = String.make !margin ' ' in
  let cl = String.length c in
  let t1 = if (cl < !margin ) then
	 String.make (!margin - cl) ' '
  else
	 "\n"^t2
  in
	 Printf.fprintf os "%s%s%s" c t1 m1;
	 List.iter (function x -> Printf.fprintf os "\n%s%s" t2 x) oth ;
	 Printf.fprintf os "\n" ;
)

let help () =
  let l = List.rev !options in
	Printf.fprintf stdout "%s\n\n" !usage;
	List.iter (pspec stdout) l;
	exit 0
  
let (mkopt : string list -> ?hide:bool -> ?arg:string -> Arg.spec ->
             string list -> unit) =
  fun ol ?(hide=false)  ?(arg="") se ml ->
  if hide then () (* some args are meaningless from lurette *) else
    let treto o = arg_options := (o, se, ""):: !arg_options in
	   List.iter treto ol ;
	   let opt_str = (String.concat ", " ol)^arg in
      options := (opt_str, ml):: !options
                      
let (mkoptab : unit -> unit) = 
  fun unit ->
    mkopt ["-sut";"--sut"]  ~arg:" <string>"
          (Arg.String (fun str -> (suts :=  str::!suts)))
          ["Add a sut"];
    mkopt ["-sut-nd";"--sut-nd"]  ~arg:" <string>"
          (Arg.String (fun str -> (suts_nd :=  str::!suts_nd)))
          ["Add a sut, but don't debug it"];
    mkopt ["-env";"--env"]  ~arg:" <string>"
          (Arg.String (fun str -> (envs :=  str::!envs)))
          ["Add an env"];
    mkopt ["-env-nd";"--env-nd"]  ~arg:" <string>"
          (Arg.String (fun str -> (envs_nd :=  str::!envs_nd)))
          ["Add an env, but don't debug it"];
    mkopt ["-oracle";"--oracle"]  ~arg:" <string>"
          (Arg.String (fun str -> (oracles :=  str::!oracles)))
          ["Add an oracle"];
    mkopt ["-oracle-nd";"--oracle-nd"]  ~arg:" <string>"
          (Arg.String (fun str -> (oracles_nd :=  str::!oracles_nd)))
          ["Add an oracle, but don't debug it"];
    mkopt ["-lurette";"--lurette"]  ~hide: !lurette
          (Arg.Unit (fun _ -> (lurette := true)))
          ["Use the lurette mode (i.e., runs everything without debugging)"];
    mkopt ["-g";"--gnuplot"] 
          (Arg.Unit (fun s -> display_gnuplot := true))
          ["in lurette mode only:  Launch gnuplot to visualize the generated rif file" ];
    mkopt ["-s";"--sim2chro"] 
          (Arg.Unit (fun s -> display_sim2chro := true))
          ["in lurette mode only:  Launch sim2chro to visualize the generated rif file" ];
    mkopt ["-l";"--test-length";"--step-number"] ~arg:" <int>"
          (Arg.Int (fun i -> (test_length := i)))
          ["in lurette mode only: Set test length (step number)"];
    mkopt ["-o";"-output";"--output";"-rif";"--rif"] ~arg:" <string>"
          (Arg.String (fun s -> output_file := s))
          ["<string>\tSet the output file name (currently,  \""^
             !output_file ^ "\")" ];
    mkopt ["-emacs";"--emacs"]  ~hide: !lurette
          (Arg.Unit (fun s -> emacs_mode := true))
          ["[experimental] Generate a  \""^rdbg_tuning_file^
             "\" file with support to open appropriate";
           "emacs buffers during sessions (won't overide it if it exists;";
           "you need to remove or rename it)" ];
    mkopt [ "-v";"-verbose";"--verbose"]
          (Arg.Unit (fun _ -> (verbose := true)))
          ["Echo the commands sent to rdbg-top"];
    mkopt ["--debug"] 
          (Arg.Unit (fun s -> drdbg := true))
          ["set on a verbose mode to (try to) understand what's going on" ];
    mkopt ["-h";"-help";"--help"]
          (Arg.Unit help)
          ["Display this help message"];
    mkopt ["--ocaml-version"] ~hide: !lurette
          (Arg.Unit(fun () -> print_string (Sys.ocaml_version); flush stdout; exit 0))
          ["Print the current ocaml version it was compiled with and exit"];
    mkopt ["-version";"--version"]
          (Arg.Unit(pversion))
          ["Print the current version and exit"]
